@@ -29,6 +29,7 @@
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-exec.h>
 #include <libgnome/gnome-help.h>
+#include <libgnome/gnome-url.h>
 #include <libgnomeui/libgnomeui.h>
 #include <libbonoboui.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
@@ -52,6 +53,7 @@
 #include "dlg-preferences.h"
 #include "dlg-rename-series.h"
 #include "dlg-scale-image.h"
+#include "dlg-crop.h"
 #include "fullscreen.h"
 #include "gconf-utils.h"
 #include "gth-pixbuf-op.h"
@@ -80,32 +82,16 @@ typedef enum {
 } WallpaperAlign;
 
 
-void 
-file_new_window_command_impl (BonoboUIComponent *uic, 
-			      gpointer           user_data, 
-			      const char        *verbname)
+static void
+open_new_window_at_location (GThumbWindow *window,
+			     const char   *location)
 {
-	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
 	GThumbWindow *new_window;
-	char         *location = NULL;
 	int           width, height;
 
-	if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
-	    && (window->dir_list->path != NULL))
-		location = g_strconcat ("file://",
-					window->dir_list->path,
-					NULL);
-	else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
-		 && (window->catalog_path != NULL))
-		location = g_strconcat ("catalog://",
-					window->catalog_path,
-					NULL);	
-
-	if (location != NULL) {
+	if (location != NULL) 
 		preferences_set_startup_location (location);
-		g_free (location);
-	}
 
 	/* Save visualization options. */
 
@@ -133,6 +119,30 @@ file_new_window_command_impl (BonoboUIComponent *uic,
 }
 
 
+void 
+file_new_window_command_impl (BonoboUIComponent *uic, 
+			      gpointer           user_data, 
+			      const char        *verbname)
+{
+	GThumbWindow *window = user_data;
+	char         *location = NULL;
+
+	if ((window->sidebar_content == GTH_SIDEBAR_DIR_LIST) 
+	    && (window->dir_list->path != NULL))
+		location = g_strconcat ("file://",
+					window->dir_list->path,
+					NULL);
+	else if ((window->sidebar_content == GTH_SIDEBAR_CATALOG_LIST) 
+		 && (window->catalog_path != NULL))
+		location = g_strconcat ("catalog://",
+					window->catalog_path,
+					NULL);	
+
+	open_new_window_at_location (window, location);
+	g_free (location);
+}
+
+
 void 
 file_close_window_command_impl (BonoboUIComponent *uic, 
 				gpointer           user_data, 
@@ -206,16 +216,29 @@ file_print_command_impl (BonoboUIComponent *uic,
 			 const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
-	char         *uri;
+	GList        *list;
+	
+	list = gth_file_view_get_file_list_selection (window->file_list->view);
+	if (list != NULL)
+		print_catalog_dlg (GTK_WINDOW (window->app), list);
+	path_list_free (list);
+}
 
-	if (window->image_path == NULL)
-		return;
 
-	uri = g_strconcat ("file://", window->image_path, NULL);
-	print_image_dlg (GTK_WINDOW (window->app), 
-			 IMAGE_VIEWER (window->viewer),
-			 uri);
-	g_free (uri);
+void 
+file_camera_import_command_impl (BonoboUIComponent *uic, 
+				 gpointer           user_data, 
+				 const gchar       *verbname)
+{
+#ifdef HAVE_LIBGPHOTO
+
+	GThumbWindow *window = user_data;
+	void (*module) (GThumbWindow *window);
+
+	if (gthumb_module_get ("dlg_photo_importer", (gpointer*) &module))
+		(*module) (window);
+
+#endif /*HAVE_LIBGPHOTO */
 }
 
 
@@ -822,26 +845,56 @@ edit_remove_from_catalog_command_impl (BonoboUIComponent *uic,
 
 
 static void
-folder_open (GThumbWindow *window,
-	     const char   *viewer,
-	     const char   *path,
-	     const char   *working_dir) 
+show_folder (GThumbWindow *window,
+	     const char   *path)
 {
 	GError *err = NULL;
-	char   *argv[5];
+	char   *uri;
 
-	argv[0] = (char*) viewer;
-	argv[1] = (char*) path;
-	argv[2] = NULL;
-	
-	if (! g_spawn_sync (working_dir, 
-			    argv, NULL, 
-			    G_SPAWN_SEARCH_PATH, 
-			    NULL, NULL, 
-			    NULL, NULL,
-			    NULL, &err))
+	uri = g_strconcat ("file://", path, NULL);
+	if (! gnome_url_show (uri, &err))
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window->app),
 						   &err);
+	g_free (uri);
+}
+
+
+void 
+edit_folder_view_command_impl (BonoboUIComponent *uic, 
+			       gpointer           user_data, 
+			       const gchar       *verbname)
+{
+	GThumbWindow  *window = user_data;
+	char          *path;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL) 
+		return;
+
+	window_go_to_directory (window, path);
+
+	g_free (path);
+}
+
+
+void 
+edit_folder_view_new_window_command_impl (BonoboUIComponent *uic, 
+					  gpointer           user_data, 
+					  const gchar       *verbname)
+{
+	GThumbWindow  *window = user_data;
+	char          *path;
+	char          *location;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL) 
+		return;
+
+	location = g_strconcat ("file://", path, NULL);
+	open_new_window_at_location (window, location);
+	g_free (location);
+
+	g_free (path);
 }
 
 
@@ -857,8 +910,7 @@ edit_folder_open_nautilus_command_impl (BonoboUIComponent *uic,
 	if (path == NULL) 
 		return;
 
-	folder_open (window, "nautilus", path, NULL);
-
+	show_folder (window, path);
 	g_free (path);
 }
 
@@ -973,7 +1025,7 @@ edit_current_folder_open_nautilus_command_impl (BonoboUIComponent *uic,
 	path = window->dir_list->path;
 	if (path == NULL) 
 		return;
-	folder_open (window, "nautilus", path, NULL);
+	show_folder (window, path);
 }
 
 
@@ -1046,6 +1098,7 @@ folder_rename (GThumbWindow *window,
 
 	} else {
 		char *old_cache_path = NULL;
+		char *old_folder_comment = NULL;
 		
 		old_cache_path = comments_get_comment_dir (old_path, TRUE, TRUE);
 		if (! path_is_dir (old_cache_path)) {
@@ -1053,14 +1106,22 @@ folder_rename (GThumbWindow *window,
 			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
 		}
 
+		old_folder_comment = comments_get_comment_filename (old_path, TRUE, TRUE);
+
 		if (rename (old_path, new_path) == 0) { 
 			char *new_cache_path;
-			
+			char *new_folder_comment;
+
 			/* Comment cache. */
-			
+
 			new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
+			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
+
 			rename (old_cache_path, new_cache_path);
+			rename (old_folder_comment, new_folder_comment);
+
 			g_free (new_cache_path);
+			g_free (new_folder_comment);
 			
 			all_windows_notify_directory_rename (old_path, new_path);
 
@@ -1075,6 +1136,7 @@ folder_rename (GThumbWindow *window,
 		}
 
 		g_free (old_cache_path);
+		g_free (old_folder_comment);
 	}
 
 	all_windows_add_monitor ();
@@ -1379,6 +1441,7 @@ folder_copy__response_cb (GObject *object,
 
 	} else {
 		char *old_cache_path = NULL;
+		char *old_folder_comment = NULL;
 
 		/* Comment cache. */
 			
@@ -1388,14 +1451,18 @@ folder_copy__response_cb (GObject *object,
 			old_cache_path = comments_get_comment_dir (old_path, FALSE, TRUE);
 		}
 
+		old_folder_comment= comments_get_comment_filename (old_path, TRUE, TRUE);
+
 		if (rename (old_path, new_path) == 0) { 
 			char *new_cache_path;
+			char *new_folder_comment;
 
 			/* moving folders on the same file system can be implemeted 
 			 * with rename, which is faster. */
 
 			new_cache_path = comments_get_comment_dir (new_path, TRUE, TRUE);
-			
+			new_folder_comment = comments_get_comment_filename (new_path, TRUE, TRUE);
+
 			if (path_is_dir (old_cache_path)) {
 				char *parent_dir;
 				
@@ -1406,7 +1473,10 @@ folder_copy__response_cb (GObject *object,
 				rename (old_cache_path, new_cache_path);
 			}
 
+			rename (old_folder_comment, new_folder_comment);
+
 			g_free (new_cache_path);
+			g_free (new_folder_comment);
 
 			all_windows_notify_directory_rename (old_path, new_path);
 
@@ -1422,6 +1492,7 @@ folder_copy__response_cb (GObject *object,
 		}
 
 		g_free (old_cache_path);
+		g_free (old_folder_comment);
 	}
 
 	g_free (dest_dir);
@@ -1514,6 +1585,110 @@ edit_current_folder_move_command_impl (BonoboUIComponent *uic,
 }
 
 
+/**/
+
+typedef struct {
+	GThumbWindow  *window;
+	GList         *file_list;
+	GList         *add_list, *remove_list;
+} FolderCategoriesData;
+
+
+static void
+edit_current_folder_categories__done (gpointer data)
+{
+	FolderCategoriesData *fcdata = data;
+	GList                *scan;
+	
+	for (scan = fcdata->file_list; scan; scan = scan->next) {
+		const char  *filename = scan->data;
+		CommentData *cdata;
+		GList       *scan2;
+
+		cdata = comments_load_comment (filename);
+		if (cdata == NULL)
+			cdata = comment_data_new ();
+		else 
+			for (scan2 = fcdata->remove_list; scan2; scan2 = scan2->next) {
+				const char *k = scan2->data;
+				comment_data_remove_keyword (cdata, k);
+			}
+
+		for (scan2 = fcdata->add_list; scan2; scan2 = scan2->next) {
+			const char *k = scan2->data;
+			comment_data_add_keyword (cdata, k);
+		}
+
+		comments_save_categories (filename, cdata);
+		comment_data_free (cdata);
+
+		dir_list_remove_directory (fcdata->window->dir_list, filename);
+		dir_list_add_directory (fcdata->window->dir_list, filename);
+	}
+
+	path_list_free (fcdata->file_list);
+	path_list_free (fcdata->add_list);
+	path_list_free (fcdata->remove_list);
+	g_free (fcdata);
+}
+
+
+static void 
+edit_current_folder_categories (GThumbWindow  *window,
+				const char    *path)
+{
+	FolderCategoriesData *fcdata;
+
+	fcdata = g_new0 (FolderCategoriesData, 1);
+	fcdata->window = window;
+	fcdata->add_list = NULL;
+	fcdata->remove_list = NULL;
+	fcdata->file_list = g_list_prepend (NULL, g_strdup (path));
+
+	dlg_choose_categories (GTK_WINDOW (window->app),
+			       fcdata->file_list,
+			       NULL,
+			       &(fcdata->add_list),
+			       &(fcdata->remove_list),
+			       edit_current_folder_categories__done,
+			       fcdata);
+}
+
+
+void 
+edit_current_folder_categories_command_impl (BonoboUIComponent *uic, 
+					     gpointer           user_data, 
+					     const gchar       *verbname)
+{
+	GThumbWindow         *window = user_data;
+	char                 *path;
+
+	path = window->dir_list->path;
+	if (path == NULL)
+		return;
+
+	edit_current_folder_categories (window, path);
+}
+
+
+void 
+edit_folder_categories_command_impl (BonoboUIComponent *uic, 
+				     gpointer           user_data, 
+				     const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	char         *path;
+
+	path = dir_list_get_selected_path (window->dir_list);
+	if (path == NULL)
+		return;
+
+	edit_current_folder_categories (window, path);
+
+	g_free (path);
+}
+
+
 void 
 edit_folder_copy_command_impl (BonoboUIComponent *uic, 
 			       gpointer           user_data, 
@@ -1937,9 +2112,9 @@ edit_current_catalog_redo_search_command_impl (BonoboUIComponent *uic,
 
 
 void 
-alter_image_rotate_command_impl (BonoboUIComponent *uic, 
-				 gpointer           user_data, 
-				 const gchar       *verbname)
+alter_image_rotate_90_command_impl (BonoboUIComponent *uic, 
+				    gpointer           user_data, 
+				    const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
@@ -1956,9 +2131,9 @@ alter_image_rotate_command_impl (BonoboUIComponent *uic,
 
 
 void 
-alter_image_rotate_cc_command_impl (BonoboUIComponent *uic, 
-				    gpointer           user_data, 
-				    const gchar       *verbname)
+alter_image_rotate_90_cc_command_impl (BonoboUIComponent *uic, 
+				       gpointer           user_data, 
+				       const gchar       *verbname)
 {
 	GThumbWindow *window = user_data;
 	ImageViewer  *viewer = IMAGE_VIEWER (window->viewer);
@@ -2188,6 +2363,16 @@ alter_image_scale_command_impl (BonoboUIComponent *uic,
 }
 
 
+void 
+alter_image_crop_command_impl (BonoboUIComponent *uic, 
+			       gpointer           user_data, 
+			       const gchar       *verbname)
+{
+	GThumbWindow *window = user_data;
+	dlg_crop (window);
+}
+
+
 void 
 view_zoom_in_command_impl (BonoboUIComponent *uic, 
 			   gpointer           user_data, 
@@ -2466,30 +2651,33 @@ set_as_wallpaper (GThumbWindow   *window,
 	GConfClient *client;
 	char        *options;
 
-	if ((image_path == NULL) || ! path_is_file (image_path))
-		return;
-
 	client = gconf_client_get_default ();
-	gconf_client_set_string (client, 
-				 "/desktop/gnome/background/picture_filename",
-				 image_path,
-				 NULL);
-	switch (align) {
-	case WALLPAPER_ALIGN_TILED:
-		options = "wallpaper";
-		break;
-	case WALLPAPER_ALIGN_CENTERED:
-		options = "centered";
-		break;
-	case WALLPAPER_ALIGN_STRETCHED:
-		options = "stretched";
-		break;
-	case WALLPAPER_ALIGN_SCALED:
-		options = "scaled";
-		break;
-	case WALLPAPER_NONE:
+
+	if ((image_path == NULL) || ! path_is_file (image_path)) 
 		options = "none";
-		break;
+
+	else {
+		gconf_client_set_string (client, 
+					 "/desktop/gnome/background/picture_filename",
+					 image_path,
+					 NULL);
+		switch (align) {
+		case WALLPAPER_ALIGN_TILED:
+			options = "wallpaper";
+			break;
+		case WALLPAPER_ALIGN_CENTERED:
+			options = "centered";
+			break;
+		case WALLPAPER_ALIGN_STRETCHED:
+			options = "stretched";
+			break;
+		case WALLPAPER_ALIGN_SCALED:
+			options = "scaled";
+			break;
+		case WALLPAPER_NONE:
+			options = "none";
+			break;
+		}
 	}
 
 	gconf_client_set_string (client, 
