@@ -2018,6 +2018,7 @@ folder_progress_update_cb (GnomeVFSAsyncHandle      *handle,
 					   info->files_total);
 		
 	} else if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
+		
 		if (fcdata->result == GNOME_VFS_OK) {
 			if (fcdata->file_op == FILE_OP_COPY)
 				all_windows_notify_directory_new (fcdata->destination); 
@@ -2198,6 +2199,7 @@ folder_copy (GThumbWindow   *window,
 	if (fcdata->include_cache) {
 		char *src_cache;
 		char *dest_cache;
+		char *src_folder_comment;
 
 		src_cache = comments_get_comment_dir (src_path, TRUE, TRUE);
 		if (! path_is_dir (src_cache)) {
@@ -2215,13 +2217,32 @@ folder_copy (GThumbWindow   *window,
 			src_list = g_list_append (src_list, 
 						  new_uri_from_path (src_cache));
 
-			if (fcdata->file_op != FILE_OP_DELETE)
+			if (fcdata->file_op != FILE_OP_DELETE) {
+				if (path_is_dir (dest_cache) && !overwrite_all)
+					rmdir_recursive (dest_cache);
+
 				dest_list = g_list_append (dest_list, 
 							   new_uri_from_path (dest_cache));
+			}
 		}
 
 		g_free (src_cache);
 		g_free (dest_cache);
+
+		/**/
+
+		src_folder_comment = comments_get_comment_filename (src_path, TRUE, TRUE);
+
+		if (path_is_file (src_folder_comment)) {
+			char *dest_folder_comment;
+			dest_folder_comment = comments_get_comment_filename (dest_path, TRUE, TRUE);
+			src_list = g_list_append (src_list, new_uri_from_path (src_folder_comment));
+			if (fcdata->file_op != FILE_OP_DELETE)
+				dest_list = g_list_append (dest_list, new_uri_from_path (dest_folder_comment));
+			g_free (dest_folder_comment);
+		}
+
+		g_free (src_folder_comment);
 	}
 
 	xfer_options = GNOME_VFS_XFER_RECURSIVE;
