@@ -33,13 +33,14 @@ extern GThumbWindow     *current_window;
 extern Preferences       preferences;
 extern FullScreen       *fullscreen;
 
-extern gint              StartInFullscreen;
-extern int               StartSlideshow;
-extern int               ViewFirstImage;
-extern int               HideSidebar;
+extern gboolean          StartInFullscreen;
+extern gboolean          StartSlideshow;
+extern gboolean          ViewFirstImage;
+extern gboolean          HideSidebar;
 extern gboolean          ExitAll;
 extern char             *ImageToDisplay;
-
+extern gboolean          FirstStart;
+extern gboolean          ImportPhotos;
 
 #define MENU_ICON_SIZE 20.0
 #define LIST_ICON_SIZE 20.0
@@ -100,4 +101,6 @@ int        get_default_folder_pixbuf_size      (GtkWidget *widget);
 
 GdkPixbuf *get_folder_pixbuf                   (double icon_size);
 
+gboolean  folder_is_film                       (const char  *full_path);
+
 #endif /* __MAIN_H__ */
